// File: renderclass.cpp

#include "loader.h"
#include "renderclass.h"

#define X__Malloc(s)    malloc(s)
#define X__Free(p)      free(p)

extern unsigned char           *ScreenL;
extern unsigned char           *ScreenR;
extern long                     X,Y;
extern BOOL                     bStereoDraw;


void Msg(TCHAR *szFormat, ...);

CRendererOpenGL::CRendererOpenGL(CMoviePlayer *pPres,
								   LPUNKNOWN pUnk, 
								   HRESULT *phr )
                                  : CBaseVideoRenderer(__uuidof(CLSID_OpenGLRenderer), 
                                    NAME("OpenGL Renderer"), pUnk, phr)
                                  , m_pCP( pPres){
    // Store and AddRef the texture for our use.
    ASSERT(phr);
    if (phr){
        if( !pPres ){
            *phr = E_INVALIDARG;
        }
        else {
            *phr = S_OK;
        }
    }
}

CRendererOpenGL::~CRendererOpenGL(){
    // Do nothing
}

HRESULT CRendererOpenGL::CheckMediaType(const CMediaType *pmt){
	// olny allows RGB video sample formats
    HRESULT   hr = E_FAIL;
    VIDEOINFO *pvi=0; 
    CheckPointer(pmt,E_POINTER);
    ASSERT( m_pCP);
    try    {
        // Reject the connection if this is not a video type
        if( *pmt->FormatType() != FORMAT_VideoInfo )  {
            return E_INVALIDARG;
        }
        pvi = (VIDEOINFO *)pmt->Format();
        if( IsEqualGUID( *pmt->Type(), MEDIATYPE_Video) )        {
            hr = S_OK;
            if( IsEqualGUID( *pmt->Subtype(), MEDIASUBTYPE_RGB24) ){
              //MessageBox(NULL,"Media OK",NULL,MB_OK);
            }
            else{
              //MessageBox(NULL,"Bad",NULL,MB_OK);
              hr = DDERR_INVALIDPIXELFORMAT;
            }
        }
        if( FAILED(hr)){
            return hr;
        }        
    }// try
    catch(...){
        Msg(TEXT("Failed to check media type in the renderer. Unhandled exception hr=0x%x"), E_UNEXPECTED);
        hr = E_UNEXPECTED;
    }   
    return hr;
}

HRESULT CRendererOpenGL::SetMediaType(const CMediaType *pmt){
    HRESULT hr = S_OK;
    VIDEOINFO *pviBmp = NULL;   // Bitmap info header
	unsigned char *S;
    try{
        // Retreive the size of this media type
        pviBmp = (VIDEOINFO *)pmt->Format();
        X = m_lVidWidth  = pviBmp->bmiHeader.biWidth;
        Y = m_lVidHeight = abs(pviBmp->bmiHeader.biHeight);
        m_lVidPitch  = (m_lVidWidth * 3 + 3) & ~(3); // We are forcing RGB24
        RECT rc; GetClientRect(m_pCP->m_hwnd,&rc);
        // there is no screen R - L&R are encoded as one double height imnage
        if(ScreenL != NULL)X__Free(ScreenL); ScreenL=NULL; 
        if((ScreenL = (unsigned char *)X__Malloc(3*X*Y)) == NULL){
          hr = E_UNEXPECTED;
		  return hr;
		}
        if(ScreenL != NULL){
           ScreenR =(ScreenL + ((Y/2) * X));
           if(bStereoDraw)Y /= 2;
        } 
	}// try
    catch(...) {
        Msg(TEXT("Failed to set media type in the renderer. Unhandled exception hr=0x%x"), E_UNEXPECTED);
        return hr;
    }
    return hr;
}

HRESULT CRendererOpenGL::DoRenderSample( IMediaSample * pSample ){
    HRESULT hr = S_OK;
    try {
      hr = m_pCP->CopyMediaSample( pSample, m_lVidPitch);
    }
    catch(...)    {
    }
    return hr;
}

void Msg(TCHAR *szFormat, ...){
    TCHAR szBuffer[1024];  // Large buffer for long filenames or URLs
    const size_t NUMCHARS = sizeof(szBuffer) / sizeof(szBuffer[0]);
    const int LASTCHAR = NUMCHARS - 1;
    va_list pArgs;
    va_start(pArgs, szFormat);
    _vsntprintf(szBuffer, NUMCHARS - 1, szFormat, pArgs);
    va_end(pArgs);
    szBuffer[LASTCHAR] = TEXT('\0');
    MessageBox(NULL, szBuffer, TEXT("Stereo Video Project\0"), 
               MB_OK | MB_ICONERROR);
}


